Package: ergmito
Version: 0.3-1
Title: Exponential Random Graph Models for Small Networks
Description: Simulation and estimation of Exponential Random Graph Models (ERGMs)
  for small networks using exact statistics as shown in Vega Yon et al. (2020)
  <DOI:10.1016/j.socnet.2020.07.005>. As a difference from the 'ergm'
  package, 'ergmito' circumvents using Markov-Chain Maximum Likelihood Estimator
  (MC-MLE) and instead uses Maximum Likelihood Estimator (MLE) to fit ERGMs
  for small networks. As exhaustive enumeration is computationally feasible for
  small networks, this R package takes advantage of this and provides tools for
  calculating likelihood functions, and other relevant functions, directly,
  meaning that in many cases both estimation and simulation of ERGMs for
  small networks can be faster and more accurate than simulation-based
  algorithms.
Depends: R (>= 3.3.0)
Authors@R: c(
      person(given = "George", family = "Vega Yon", role = c("cre","aut"),
      email = "g.vegayon@gmail.com", comment = c(ORCID = "0000-0002-3171-0844")),
      person(given = "Kayla", family = "de la Haye", role = c("ths"),
      comment = c(ORCID = "0000-0002-2536-7701")),
      person("Army Research Laboratory and the U.S. Army Research Office",
      role = "fnd", comment = "Grant Number W911NF-15-1-0577")
      )
RoxygenNote: 7.2.3
Encoding: UTF-8
Imports: ergm, network, MASS, Rcpp, texreg, stats, parallel, utils,
        methods, graphics
LinkingTo: Rcpp, RcppArmadillo
License: MIT + file LICENSE
Suggests: covr, sna, lmtest, fmcmc, coda, knitr, rmarkdown, tinytest
VignetteBuilder: knitr
LazyData: true
URL: https://muriteams.github.io/ergmito/
BugReports: https://github.com/muriteams/ergmito/issues
Language: en-US
NeedsCompilation: yes
Packaged: 2023-06-13 19:39:54 UTC; george
Author: George Vega Yon [cre, aut] (<https://orcid.org/0000-0002-3171-0844>),
  Kayla de la Haye [ths] (<https://orcid.org/0000-0002-2536-7701>),
  Army Research Laboratory and the U.S. Army Research Office [fnd] (Grant
    Number W911NF-15-1-0577)
Maintainer: George Vega Yon <g.vegayon@gmail.com>
Repository: CRAN
Date/Publication: 2023-06-14 10:42:05 UTC
Built: R 4.4.0; aarch64-apple-darwin22.6.0; 2024-04-30 06:31:00 UTC; unix
