# Generated by roxygen2: do not edit by hand

S3method("[",ergmito_sampler)
S3method(as_adjmat,formula)
S3method(as_adjmat,list)
S3method(as_adjmat,matrix)
S3method(as_adjmat,network)
S3method(coef,ergmito)
S3method(count_stats,formula)
S3method(count_stats,list)
S3method(ergmito_boot,ergmito)
S3method(ergmito_boot,formula)
S3method(exact_gradient,default)
S3method(exact_gradient,ergmito_ptr)
S3method(exact_loglik,default)
S3method(exact_loglik,ergmito_ptr)
S3method(formula,ergmito)
S3method(geodesic,list)
S3method(geodesic,matrix)
S3method(geodesic,network)
S3method(induced_submat,list)
S3method(induced_submat,matrix)
S3method(induced_submat,network)
S3method(is_directed,default)
S3method(is_directed,ergmito)
S3method(is_directed,formula)
S3method(is_directed,list)
S3method(is_directed,network)
S3method(logLik,ergmito)
S3method(matrix_to_network,list)
S3method(matrix_to_network,matrix)
S3method(nedges,ergmito)
S3method(nedges,formula)
S3method(nedges,list)
S3method(nedges,matrix)
S3method(nedges,network)
S3method(new_rergmito,formula)
S3method(nnets,ergmito)
S3method(nnets,formula)
S3method(nnets,list)
S3method(nnets,matrix)
S3method(nnets,network)
S3method(nobs,ergmito)
S3method(nvertex,ergmito)
S3method(nvertex,formula)
S3method(nvertex,list)
S3method(nvertex,matrix)
S3method(nvertex,network)
S3method(plot,ergmito)
S3method(plot,ergmito_benchmark)
S3method(plot,ergmito_gof)
S3method(predict,ergmito)
S3method(print,ergmito)
S3method(print,ergmito_benchmark)
S3method(print,ergmito_boot)
S3method(print,ergmito_gof)
S3method(print,ergmito_loglik)
S3method(print,ergmito_sampler)
S3method(print,ergmito_summary)
S3method(same_dist,matrix)
S3method(same_dist,network)
S3method(simulate,ergmito)
S3method(str,ergmito_sampler)
S3method(summary,ergmito)
S3method(vcov,ergmito)
export(AVAILABLE_STATS)
export(as_adjmat)
export(benchmarkito)
export(blockdiagonalize)
export(count_stats)
export(ergm_blockdiag)
export(ergmito)
export(ergmito_boot)
export(ergmito_formulae)
export(exact_gradient)
export(exact_hessian)
export(exact_loglik)
export(extract.ergmito)
export(geodesic)
export(geodesita)
export(gof_ergmito)
export(induced_submat)
export(is_directed)
export(matrix_to_network)
export(nedges)
export(new_rergmito)
export(nnets)
export(nvertex)
export(powerset)
export(rbernoulli)
export(same_dist)
export(splitnetwork)
importFrom(MASS,ginv)
importFrom(Rcpp,sourceCpp)
importFrom(ergm,ergm)
importFrom(ergm,ergm.allstats)
importFrom(graphics,boxplot)
importFrom(graphics,image)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,title)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,setRefClass)
importFrom(network,get.vertex.attribute)
importFrom(network,is.network)
importFrom(network,list.vertex.attributes)
importFrom(network,network.size)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,makePSOCKcluster)
importFrom(parallel,stopCluster)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,formula)
importFrom(stats,logLik)
importFrom(stats,model.matrix)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(texreg,extract)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,ls.str)
useDynLib(ergmito, .registration = TRUE)
